@@ -23,13 +23,60 @@ from .. import reset_db_sequence
 
 
 # --- 初始化环境 ---
-from .admin_env_init import CHANGE_LIST_HTML_PATH
+from .admin_env_init import CHANGE_LIST_HTML_PATH, TEMPDIR, BD_USE_GUARDIAN, CHANGE_FORM_TEMPLATE
 from tqdm import tqdm
 from pandas._libs.tslibs.timestamps import Timestamp
 from bddjango import get_base_model, get_model_max_id_in_db, old_get_model_max_id_in_db
 from django.contrib.admin.helpers import ACTION_CHECKBOX_NAME
 
 
+def _remove_temp_file(tempdir=TEMPDIR, MAX_TEMPS=5):
+    """
+    清理缓存, 清空tempdir下的所有文件
+    :param tempdir: 文件路径
+    :param MAX_TEMPS: 最大缓存数量
+    """
+    fpath_ls = os.listdir(tempdir)
+    temps = len(fpath_ls)
+
+    if temps < MAX_TEMPS:
+        print(f'...缓存还足够, 不用清理... 缓存容量: {temps}/{MAX_TEMPS}')
+        return
+    tt = Time()
+
+    tt.sleep(1)
+    print('*************** 开始清理缓存 *************')
+    for fpath in fpath_ls:
+        i = 0
+        tt.__init__()
+        while tt.during(5):
+            i += 1
+            dirpath = os.path.join(tempdir, fpath)
+            try:
+                os.remove(dirpath)
+                print(f"~~~ success: 移除文件[{dirpath}]成功! -- 第[{i}]次")
+                break
+            except:
+                print(f"** 第[{i}]次移除文件[{dirpath}]失败...")
+                tt.sleep(1)
+                if i > 5:
+                    print(f"======== Warning: 移除文件[{dirpath}]失败!")
+    print('*************** 缓存清理完毕 *************')
+
+
+def remove_temp_file(tempdir=TEMPDIR, MAX_TEMPS = 5):
+    """
+    当大于MAX_TEMPS时启动临时文件清理程序
+    """
+    temps = len(os.listdir(tempdir))
+
+    if temps > MAX_TEMPS:
+        t1 = threading.Thread(target=_remove_temp_file, args=(tempdir, MAX_TEMPS, ))
+        t1.start()
+    else:
+        print(f'...缓存还足够, 不用清理... 缓存文件: {temps}/{MAX_TEMPS}')
+    return
+
 class IDAdmin(admin.ModelAdmin):
     """
     * 保存时自动处理id, 解决postgre在批量导入数据后的主键冲突问题.
@@ -59,6 +106,36 @@ class IDAdmin(admin.ModelAdmin):
             obj.save()
 
 
+if BD_USE_GUARDIAN:
+    from guardian.admin import GuardedModelAdmin
+
+
+    class BaseGuarDianAdmin(GuardedModelAdmin):
+        """
+        * 保存时自动处理id, 解决postgre在批量导入数据后的主键冲突问题.
+
+        - 该方法仅对admin界面手动保存时调用的save_model()方法生效, 不影响obj.save()方法效率.
+        """
+        change_form_template = CHANGE_FORM_TEMPLATE
+
+        def save_model(self, request, obj, form, change):
+            """
+            参数change分辨保存or修改.
+            若为保存, 则model的id值自动更新为数据库中最大id+1.
+            """
+            try:
+                obj.save()
+            except Exception as e:
+                msg = f'可能为pgsql的id引起的错误:' + str(e)
+
+                print(msg)
+                reset_db_sequence(obj)
+                obj.save()
+
+
+    IDAdmin = BaseGuarDianAdmin
+
+
 # --- 处理df中的特殊格式
 def conv_date_field_str_format(ts):
     """
@@ -172,36 +249,91 @@ class ExportExcelMixin:
 
     def export_as_excel(self, request, queryset=None, model=None):
         if model is None:
+            # 如果没有指定model, 则采用默认的model, 并导出全部
             meta = self.model._meta
         else:
             meta = model._meta
+            queryset = model.objects.all()
 
+        # md = get_base_model(queryset)
         field_names = [field.name for field in meta.fields]
         verbose_names = [field.verbose_name for field in meta.fields]
+        conv_name_to_verbose_dc = dict(zip(field_names, verbose_names))
 
-        response = HttpResponse(content_type='application/msexcel')
-        filename = urlquote(f"{meta.verbose_name}.xlsx")
-        response['Content-Disposition'] = f'attachment; filename={filename}'
-        wb = Workbook()
-        ws = wb.active
+        if model is None:
+            """
+            导出全部的时候, 大文件用流媒体传输, 并分批次读入, 防止超过云服务器带宽.
+            """
+            from django.http import StreamingHttpResponse
+            from django_pandas import io
+            from bdtime import tt
+            from django.db import models as m
+            print('~~~~~~~~~~~~ queryset.count():', queryset.count())
+            qs: m.QuerySet = queryset
 
-        if self.export_asc:
-            if queryset.count() and hasattr(queryset[0], 'id'):
-                queryset = queryset.order_by('id')
-        ws.append(verbose_names)
+            tt.__init__()
 
-        total = queryset.count()
-        tq = tqdm(total=total)
-        for obj in queryset:
-            tq.update(1)
-            data = [f'{getattr(obj, field)}' for field in field_names]
-            try:
-                ws.append(data)
-            except Exception as e:
-                print(data)
-                raise e
-        wb.save(response)
-        return response
+            print('开始读入sql...')
+            qs: m.QuerySet
+            # import pandas as pd
+            # from django.db import connection
+            # df = pd.concat([df for df in pd.read_sql(f'SELECT * FROM {meta.db_table}', connection, chunksize=100)], axis=0)
+            df = io.read_frame(qs=qs)
+            df.columns = [conv_name_to_verbose_dc.get(col) for col in df.columns]
+            print('df读入成功: ', tt.now(2))
+
+            print('开始保存为excel...', tt.now(2))
+            time_str = datetime.datetime.now().strftime("%Y%m%d_%H%M%S")
+            tmp_file_name = f"{meta.verbose_name}_{time_str}.xlsx"
+            fpath = os.path.join(TEMPDIR, tmp_file_name)
+            # df.to_excel(fpath, index=False, encoding='unicode_escape')       # unicode_escape
+            df.to_excel(fpath, index=False)       # unicode_escape
+            print('保存为Excel成功!', tt.now(2))
+            del df
+
+            # big file download
+            def file_iterator(file_name, open_model='rb', chunk_size=512):
+                with open(file_name, open_model) as f:      # , encoding=encoding
+                    while True:
+                        c = f.read(chunk_size)
+                        if c:
+                            yield c
+                        else:
+                            break
+
+            print('write response', tt.now(2))
+            response = StreamingHttpResponse(file_iterator(fpath))
+            # response['Content-Type'] = 'application/octet-stream'
+            filename = urlquote(f"{meta.verbose_name}.xlsx")
+            response['Content-Type'] = 'application/msexcel'
+            response['Content-Disposition'] = 'attachment;filename="{0}"'.format(filename)
+
+            print('return response', tt.now(2))
+            return response
+        else:
+            response = HttpResponse(content_type='application/msexcel')
+            filename = urlquote(f"{meta.verbose_name}.xlsx")
+            response['Content-Disposition'] = f'attachment; filename={filename}'
+            wb = Workbook()
+            ws = wb.active
+
+            if self.export_asc:
+                if queryset.count() and hasattr(queryset[0], 'id'):
+                    queryset = queryset.order_by('id')
+            ws.append(verbose_names)
+
+            total = queryset.count()
+            tq = tqdm(total=total)
+            for obj in queryset:
+                tq.update(1)
+                data = [f'{getattr(obj, field)}' for field in field_names]
+                try:
+                    ws.append(data)
+                except Exception as e:
+                    print(data)
+                    raise e
+            wb.save(response)
+            return response
 
     export_as_excel.short_description = "导出所选数据"
     # export_as_excel.acts_on_all = True
@@ -374,7 +506,9 @@ class ImportAdmin(IDAdmin):
 
     def export_all_excel(self, request):
         # self.message_user(request, "成功导出全部数据为csv文件")
-        return ExportExcelMixin().export_as_excel(request, queryset=self.model.objects.all(), model=self.model)
+        ret = ExportExcelMixin().export_as_excel(request, queryset=self.model.objects.all(), model=self.model)
+        self.remove_temp_file(TEMPDIR)
+        return ret
 
     def get_urls(self):
         my_urls = [
@@ -385,45 +519,7 @@ class ImportAdmin(IDAdmin):
         return my_urls + super().get_urls()
 
     def remove_temp_file(self, tempdir):
-        # 内存清理
-        temps = len(os.listdir(tempdir))
-        MAX_TEMPS = 5
-        if temps > MAX_TEMPS:
-            t1 = threading.Thread(target=ImportAdmin._remove_temp_file, args=(tempdir, MAX_TEMPS, ))
-            t1.start()
-        else:
-            print(f'...excel缓存还足够, 不用清理... 缓存容量: {temps}/{MAX_TEMPS}')
-        return
-
-    @staticmethod
-    def _remove_temp_file(tempdir, MAX_TEMPS=5):
-        # --- 清理缓存, 清空tempdir下的所有文件
-        fpath_ls = os.listdir(tempdir)
-        temps = len(fpath_ls)
-
-        if temps < MAX_TEMPS:
-            print(f'...缓存还足够, 不用清理... 缓存容量: {temps}/{MAX_TEMPS}')
-            return
-        tt = Time()
-
-        tt.sleep(1)
-        print('*************** 开始清理缓存 *************')
-        for fpath in fpath_ls:
-            i = 0
-            tt.__init__()
-            while tt.during(5):
-                i += 1
-                dirpath = os.path.join(tempdir, fpath)
-                try:
-                    os.remove(dirpath)
-                    print(f"~~~ success: 移除文件[{dirpath}]成功! -- 第[{i}]次")
-                    break
-                except:
-                    print(f"** 第[{i}]次移除文件[{dirpath}]失败...")
-                    tt.sleep(1)
-                    if i > 5:
-                        print(f"======== Warning: 移除文件[{dirpath}]失败!")
-        print('*************** 缓存清理完毕 *************')
+        return remove_temp_file(tempdir)
 
 
 class CsvImportExportAdmin(ImportAdmin, ExportCsvMixin):
@@ -493,8 +589,6 @@ class ListDisplayAdmin(ExcelImportExportAdmin):
     list_display = '__all__'        # 默认展示全部
     stop_field_ls = []              # 停用字段
     move_id_to_tail = False      # id移到最后一列去
-    # search_fields = ['id', ]
-    # list_per_page = 10
 
     origin_list = False     # 展示原str
 
